/** @file
 *
 * X11 clipboard format viewer
 */

/*
 * Copyright (C) 2006-2007 Oracle Corporation
 *
 * This file is part of VirtualBox Open Source Edition (OSE), as
 * available from http://www.virtualbox.org. This file is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software
 * Foundation, in version 2 as it comes in the "COPYING" file of the
 * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 */

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/Xproto.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <stdlib.h>

/** Convert an atom name string to an X11 atom, looking it up in a cache
 * before asking the server */
static Atom clipGetAtom(Widget widget, const char *pszName)
{
    Atom retval = None;
    XrmValue nameVal, atomVal;
    nameVal.addr = (char *) pszName;
    nameVal.size = strlen(pszName);
    atomVal.size = sizeof(Atom);
    atomVal.addr = (char *) &retval;
    XtConvertAndStore(widget, XtRString, &nameVal, XtRAtom, &atomVal);
    return retval;
}

/**
 * Print out information about available data formats, based on the "targets"
 * information obtained from the X11 clipboard.
 * @note  callback for XtGetSelectionValue
 */
static void clipConvertX11Targets(Widget, XtPointer pClientData,
                                  Atom * /* selection */, Atom *atomType,
                                  XtPointer pValue, long unsigned int *pcLen,
                                  int *piFormat)
{
    Widget widget = (Widget) pClientData;
    if (*atomType == XT_CONVERT_FAIL)  /* timeout */
       printf("Time out getting clipboard format data.\n");
    else if (pValue == NULL)  /* No data available */
       printf("No clipboard format data available.\n");
    else
    {
        Atom *pTargets = (Atom *) pValue;
        for (unsigned i = 0; i < *pcLen; ++i)
        {
            char *pszName = XGetAtomName(XtDisplay(widget), pTargets[i]);
            printf("Found clipboard format: %s\n", pszName);
        }
        XtFree(reinterpret_cast<char *>(pValue));
    }
    XtAppSetExitFlag(XtWidgetToApplicationContext(widget));
}

int main()
{
    int cArgc = 0;
    char *pcArgv = 0;
    XtToolkitInitialize();
    XtAppContext ctx = XtCreateApplicationContext();
    Display *pDisplay = XtOpenDisplay(ctx, 0, 0, "ViewClipFormats", 0, 0,
                                        &cArgc, &pcArgv);
    if (!pDisplay)
    {
        printf("Unable to access the X11 window system.  Exiting.\n");
        exit(1);
    }
    Widget widget = XtVaAppCreateShell(0, "ViewClipFormats",
                                          applicationShellWidgetClass,
                                          pDisplay, XtNwidth, 1, XtNheight,
                                          1, NULL);
    if (!widget)
    {
        printf("Failed to create a hidden window for the clipboard format viewer.  Exiting.\n");
        exit(1);
    }
    XtSetMappedWhenManaged(widget, false);
    XtRealizeWidget(widget);
    XtGetSelectionValue(widget, clipGetAtom(widget, "CLIPBOARD"),
                        clipGetAtom(widget, "TARGETS"), clipConvertX11Targets,
                        widget, CurrentTime);
    XtAppMainLoop(ctx);
}
